@@ -16,9 +16,21 @@ class MopidyException(Exception):
         self._message = message
 
 
+class BackendError(MopidyException):
+    pass
+
+
 class ExtensionError(MopidyException):
     pass
 
 
+class FrontendError(MopidyException):
+    pass
+
+
+class MixerError(MopidyException):
+    pass
+
+
 class ScannerError(MopidyException):
     pass
